@@ -34,7 +34,7 @@ class UpgradeCommand(commands.Command):
     """A class containing methods needed by the cli to execute the
     update command.
     """
-    aliases = ('upgrade', 'update', 'upgrade-to', 'update-to')
+    aliases = ('upgrade', 'update', 'upgrade-to', 'update-to', 'up')
     summary = _('upgrade a package or packages on your system')
 
     @staticmethod
@@ -55,7 +55,8 @@ class UpgradeCommand(commands.Command):
         demands.resolving = True
         demands.root_user = True
         commands._checkGPGKey(self.base, self.cli)
-        commands._checkEnabledRepo(self.base, self.opts.filenames)
+        if not self.opts.filenames:
+            commands._checkEnabledRepo(self.base)
         self.upgrade_minimal = None
         self.all_security = None
 
@@ -80,9 +81,13 @@ class UpgradeCommand(commands.Command):
             for pkg_spec in self.opts.pkg_specs:
                 try:
                     self.base.upgrade(pkg_spec)
+                except dnf.exceptions.PackagesNotInstalledError:
+                    logger.info(_('No match for argument: %s'),
+                                self.base.output.term.bold(pkg_spec))
                 except dnf.exceptions.MarkingError as e:
                     logger.info(_('No match for argument: %s'),
                                  self.base.output.term.bold(pkg_spec))
+                    self.base._report_icase_hint(pkg_spec)
                 else:
                     done = True
 
