@@ -3,6 +3,7 @@
 # This file is part of dnf.
 #
 # Copyright 2015 (C) Igor Gnatenko <i.gnatenko.brain@gmail.com>
+# Copyright 2016 (C) Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +22,7 @@
 
 import dnf.exceptions
 import dnf.cli
-import re
+import dnf.cli.commands.clean
 import sys
 
 
@@ -35,12 +36,12 @@ class RemoveCompletionCommand(dnf.cli.commands.remove.RemoveCommand):
     def __init__(self, args):
         super(RemoveCompletionCommand, self).__init__(args)
 
-    def configure(self, args):
+    def configure(self):
         self.cli.demands.root_user = False
         self.cli.demands.sack_activation = True
 
-    def run(self, args):
-        for pkg in ListCompletionCommand.installed(self.base, args[0]):
+    def run(self):
+        for pkg in ListCompletionCommand.installed(self.base, self.opts.pkg_specs):
             print(str(pkg))
 
 
@@ -48,14 +49,14 @@ class InstallCompletionCommand(dnf.cli.commands.install.InstallCommand):
     def __init__(self, args):
         super(InstallCompletionCommand, self).__init__(args)
 
-    def configure(self, args):
+    def configure(self):
         self.cli.demands.root_user = False
         self.cli.demands.available_repos = True
         self.cli.demands.sack_activation = True
 
-    def run(self, args):
-        installed =  listpkg_to_setstr(ListCompletionCommand.installed(self.base, args[0]))
-        available = listpkg_to_setstr(ListCompletionCommand.available(self.base, args[0]))
+    def run(self):
+        installed =  listpkg_to_setstr(ListCompletionCommand.installed(self.base, self.opts.pkg_specs))
+        available = listpkg_to_setstr(ListCompletionCommand.available(self.base, self.opts.pkg_specs))
         for pkg in (available - installed):
             print(str(pkg))
 
@@ -64,14 +65,14 @@ class ReinstallCompletionCommand(dnf.cli.commands.reinstall.ReinstallCommand):
     def __init__(self, args):
         super(ReinstallCompletionCommand, self).__init__(args)
 
-    def configure(self, args):
+    def configure(self):
         self.cli.demands.root_user = False
         self.cli.demands.available_repos = True
         self.cli.demands.sack_activation = True
 
-    def run(self, args):
-        installed =  listpkg_to_setstr(ListCompletionCommand.installed(self.base, args[0]))
-        available = listpkg_to_setstr(ListCompletionCommand.available(self.base, args[0]))
+    def run(self):
+        installed =  listpkg_to_setstr(ListCompletionCommand.installed(self.base, self.opts.packages))
+        available = listpkg_to_setstr(ListCompletionCommand.available(self.base, self.opts.packages))
         for pkg in (installed & available):
             print(str(pkg))
 
@@ -79,8 +80,9 @@ class ListCompletionCommand(dnf.cli.commands.ListCommand):
     def __init__(self, args):
         super(ListCompletionCommand, self).__init__(args)
 
-    def run(self, args):
-        subcmds = self.__class__.__base__.usage[1:-1].split("|")[1:]
+    def run(self):
+        subcmds = self.pkgnarrows
+        args = self.opts.packages
         if args[0] not in subcmds:
             print("\n".join(filter_list_by_kw(args[1], subcmds)))
         else:
@@ -112,7 +114,8 @@ class RepoListCompletionCommand(dnf.cli.commands.repolist.RepoListCommand):
     def __init__(self, args):
         super(RepoListCompletionCommand, self).__init__(args)
 
-    def run(self, args):
+    def run(self):
+        args = self.opts.extcmds
         if args[0] == "enabled":
             print("\n".join(filter_list_by_kw(args[1], [r.id for r in self.base.repos.iter_enabled()])))
         elif args[0] == "disabled":
@@ -123,13 +126,13 @@ class UpgradeCompletionCommand(dnf.cli.commands.upgrade.UpgradeCommand):
     def __init__(self, args):
         super(UpgradeCompletionCommand, self).__init__(args)
 
-    def configure(self, args):
+    def configure(self):
         self.cli.demands.root_user = False
         self.cli.demands.available_repos = True
         self.cli.demands.sack_activation = True
 
-    def run(self, args):
-        for pkg in ListCompletionCommand.updates(self.base, args[0]):
+    def run(self):
+        for pkg in ListCompletionCommand.updates(self.base, self.opts.pkg_specs):
             print(str(pkg))
 
 
@@ -137,13 +140,13 @@ class DowngradeCompletionCommand(dnf.cli.commands.downgrade.DowngradeCommand):
     def __init__(self, args):
         super(DowngradeCompletionCommand, self).__init__(args)
 
-    def configure(self, args):
+    def configure(self):
         self.cli.demands.root_user = False
         self.cli.demands.available_repos = True
         self.cli.demands.sack_activation = True
 
-    def run(self, args):
-        for pkg in ListCompletionCommand.available(self.base, args[0]).downgrades():
+    def run(self):
+        for pkg in ListCompletionCommand.available(self.base, self.opts.package).downgrades():
             print(str(pkg))
 
 
@@ -151,9 +154,9 @@ class CleanCompletionCommand(dnf.cli.commands.clean.CleanCommand):
     def __init__(self, args):
         super(CleanCompletionCommand, self).__init__(args)
 
-    def run(self, args):
-        subcmds = self.__class__.__base__.usage[1:-1].split("|")[1:]
-        print("\n".join(filter_list_by_kw(args[1], subcmds)))
+    def run(self):
+        subcmds = dnf.cli.commands.clean._CACHE_TYPES.keys()
+        print("\n".join(filter_list_by_kw(self.opts.type[1], subcmds)))
 
 
 class HistoryCompletionCommand(dnf.cli.commands.HistoryCommand):
@@ -161,17 +164,16 @@ class HistoryCompletionCommand(dnf.cli.commands.HistoryCommand):
         super(HistoryCompletionCommand, self).__init__(args)
 
     def run(self, args):
-        subcmds = self.__class__.__base__.usage[1:-1].split("|")[1:]
+        subcmds = self.__class__.__base__.usage[1:-1].split("|")
         if args[0] not in subcmds:
-            print("\n".join(filter_list_by_kw(args[1], subcmds)))
+            print("\n".join(filter_list_by_kw(self.opts.tid, subcmds)))
 
 
 def main(args):
     base = dnf.cli.cli.BaseCli()
     cli = dnf.cli.Cli(base)
     if args[0] == "_cmds":
-        base.plugins.load(base.conf.pluginpath, [])
-        base.plugins.run_init(base, cli)
+        base.init_plugins([], [], cli)
         print("\n".join(filter_list_by_kw(args[1], cli.cli_commands)))
         return
     cli.cli_commands.clear()
